--
-- mogliBasics
-- This is the specialization for mogliBasics
--




mogliBasics = {}
local mogliBasics_directory = g_currentModDirectory

function mogliBasics.newSubclass()
	local subclass = {}
	setmetatable( subclass, { __index = function (table, key) return mogliBasics[key] end } )
	return subclass
end

function mogliBasics.globalsReset( createIfMissing )
end

function mogliBasics.globalsLoad( file , rootTag, globals )	

	local xmlFile = loadXMLFile( "mogliBasics", file, rootTag )
	for name,value in pairs(globals) do
		local tp = getXMLString(xmlFile, rootTag.."." .. name .. "#type")
		if     tp == nil then
--			print(file..": "..name.." = nil")
		elseif tp == "bool" then
			local bool = getXMLBool( xmlFile, rootTag.."." .. name .. "#value" )
			if bool ~= nil then
				--if bool then globals[name] = 1 else globals[name] = 0 end
				globals[name] = bool
			end
--			print(file..": "..name.." = "..tostring(globals[name]))
		elseif tp == "float" then
			local float = getXMLFloat( xmlFile, rootTag.."." .. name .. "#value" )
			if float ~= nil then globals[name] = float end
--			print(file..": "..name.." = "..tostring(globals[name]))
		elseif tp == "int" then
			local int = getXMLInt( xmlFile, rootTag.."." .. name .. "#value" )
			if int ~= nil then globals[name] = int end
--			print(file..": "..name.." = "..tostring(globals[name]))
		else
			print(file..": "..name..": invalid XML type : "..tp)
		end
	end
end

function mogliBasics.getText(id)
	if id == nil then
		return "nil";
	end;
	
	if g_i18n:hasText( id ) then
	  return g_i18n:getText( id )
	end
	
	return id
end;

function mogliBasics.normalizeAngle( angle )
	local normalizedAngle = angle
	if angle > math.pi then
		normalizedAngle = angle - math.pi - math.pi
	elseif angle <= -math.pi then
		normalizedAngle = angle + math.pi + math.pi
	end
	return normalizedAngle
end

function mogliBasics.prerequisitesPresent(specializations)
  return true
end

function mogliBasics:load(xmlFile)
-- should always be overwritten
	mogliBasics.registerState( self, "mogliBasicsDummy", false, mogliBasics.debugEvent )
end

function mogliBasics:delete()
end

function mogliBasics:mouseEvent(posX, posY, isDown, isUp, button)
end

function mogliBasics:keyEvent(unicode, sym, modifier, isDown)
end

function mogliBasics:update(dt)
end

function mogliBasics:updateTick(dt)	
end

function mogliBasics:readStream(streamId, connection)
end

function mogliBasics:writeStream(streamId, connection)
end

function mogliBasics:draw()	
end  
       
function mogliBasics:getSaveAttributesAndNodes(nodeIdent)
	local attributes = ""
	return attributes
end;

function mogliBasics:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	return BaseMission.VEHICLE_LOAD_OK;
end

function mogliBasics:registerState( level1, default, handler )
	if self.mogliSetState ~= mogliBasics.mogliSetState then
		self.mogliSetState = mogliBasics.mogliSetState
  end
	if self.mogliOnSetState == nil then
		self.mogliOnSetState = {}
	end
	
	self[level1] = default
	if handler ~= nil then
		self.mogliOnSetState[level1] = handler
	end		
end

function mogliBasics:mogliSetState(level1, value, noEventSend)
	
	if self[level1] == nil or self[level1] ~= value then
	
		local valueEvent, valueNew 
		
		if value== true or value=="true" then
			valueEvent = "true"
		elseif value==false or value=="false" then
			valueEvent = "false"
		elseif value==nil then
			valueEvent = "nil"
		elseif type(value)=="string" then
			valueEvent = '"'..value..'"'		
		elseif value ~= "true" and value ~= "false" then
			valueEvent = string.format("%f", value)
		else
			valueEvent = value 
		end
		
		if valueEvent== "true" then
			valueNew = true
		elseif valueEvent=="false" then
			valueNew = false
		elseif valueEvent=="nil" then
			valueNew = nil
		elseif  string.len(valueEvent)>=2 
		    and string.sub(valueEvent,1,1)=='"'
		    and string.sub(valueEvent,-1,-1)=='"' then
			valueNew = string.sub(valueEvent,2,-2)
		elseif valueEvent ~= true and valueEvent ~= false then
			valueNew = tonumber(valueEvent)
		else
			valueNew = valueEvent 
		end
	
		if noEventSend == nil or noEventSend == false then
			if g_server ~= nil then 
				g_server:broadcastEvent(mogliBasicsEvent:new(self, level1, valueEvent), nil, nil, object) 
			else
				g_client:getServerConnection():sendEvent(mogliBasicsEvent:new(self, level1, valueEvent)) 
			end 
		end 						
	
		if self.mogliOnSetState[level1] ~= nil then
			local noEventSend2 = true 
			if self.isServer then
				noEventSend2 = noEventSend 
			end 
			
			local state, message = pcall( self.mogliOnSetState[level1], self, self[level1], valueNew, noEventSend2 )
			if not state then
				print("Error: "..tostring(message)) 
				mogliBasics.debugEvent( self, self[level1], valueNew, noEventSend ) 
				self[level1] = valueNew 	
			end
		else
			self[level1]= valueNew 	
		end 	
	end 	
end 

function mogliBasics:debugEvent( old, new, noEventSend )
	local i = 2 
	local info 
	print("------------------------------------------------------------------------") 
	while i <= 10 do
		info = debug.getinfo(i) 
		if info == nil then break end
		print(string.format("%i: %s (%i): %s", i, info.short_src, Utils.getNoNil(info.currentline,0), Utils.getNoNil(info.name,"<???>"))) 
		i = i + 1 
	end
	if info ~= nil and info.name ~= nil and info.currentline ~= nil then
		print("...") 
	end
	print("------------------------------------------------------------------------") 
	print(tostring(old).." "..tostring(new).." "..tostring(noEventSend)) 
end 


mogliBasicsEvent = {} 
mogliBasicsEvent_mt = Class(mogliBasicsEvent, Event) 

InitEventClass(mogliBasicsEvent, "mogliBasicsEvent") 

 
function mogliBasicsEvent:emptyNew()
	local self = Event:new(mogliBasicsEvent_mt) 
	self.className="mogliBasicsEvent" 
	return self 
end 

function mogliBasicsEvent:new(object, level1, value)
	local self = mogliBasicsEvent:emptyNew() 
  self.object = object 
  self.level1 = level1 
  self.value = value 
	return self 
end 

function mogliBasicsEvent:readStream(streamId, connection)
	--both clients and server can receive this event
  local id = streamReadInt32(streamId) 
  self.object = networkGetObject(id) 
  self.level1 = streamReadString(streamId) 
  self.value = streamReadString(streamId) 
  self:run(connection) 
end 

function mogliBasicsEvent:writeStream(streamId, connection)
	--both clients and server can send this event
  streamWriteInt32(streamId, networkGetObjectId(self.object)) 
  streamWriteString(streamId, self.level1 ) 
  streamWriteString(streamId, self.value ) 
end 

function mogliBasicsEvent:run(connection)
	----both clients and server can "run" this event (after reading it)	
	self.object:mogliSetState(self.level1, self.value, true) 
  if not connection:getIsServer() then  
    g_server:broadcastEvent(mogliBasicsEvent:new(self.object, self.level1, self.value), nil, connection, self.object) 
  end 
end 